import networkx as nx


def serialize_individuals_and_connections_old(individuals_qs):
    serialized_individuals = []
    serialized_connections = []

    for individual in individuals_qs:
        serialized_individual = {
            "id": individual.id,
            "immune_system": individual.immune_system,
        }
        serialized_individuals.append(serialized_individual)

        for connection in individual.individual_connections.all():
            serialized_connections.append(
                {
                    "source_id": connection.individual_id,
                    "target_id": connection.connected_individual_id,
                    "weight": connection.weight,
                }
            )

    return serialized_individuals, serialized_connections


def serialize_individuals_and_connections(individuals_qs):
    serialized_individuals = [
        {
            'id': individual.id,
            'immune_system': individual.immune_system,
        }
        for individual in individuals_qs
    ]

    serialized_connections = [
        {
            'source_id': connection.individual_id,
            'target_id': connection.connected_individual_id,
            'weight': connection.weight,
        }
        for individual in individuals_qs
        for connection in individual.individual_connections.all()
    ]

    return serialized_individuals, serialized_connections


def deserialize_individuals_and_connections(serialized_individuals, serialized_connections, target_generation):
    from simulations.models import Individual, Connection
    # Create a list of new individual instances
    new_individuals = [
        Individual(
            generation=target_generation,
            immune_system=serialized_individual['immune_system']
        )
        for serialized_individual in serialized_individuals
    ]

    # Bulk create individuals
    created_individuals = Individual.objects.bulk_create(new_individuals)

    # Create a mapping of old individual IDs to new individual instances
    id_mapping = {
        serialized_individual['id']: new_individual
        for serialized_individual, new_individual in zip(serialized_individuals, created_individuals)
    }

    # Create a list of new connection instances
    new_connections = [
        Connection(
            individual=id_mapping[serialized_connection['source_id']],
            connected_individual=id_mapping[serialized_connection['target_id']],
            weight=serialized_connection['weight']
        )
        for serialized_connection in serialized_connections
    ]

    # Bulk create connections
    Connection.objects.bulk_create(new_connections)



def create_networkx_graph_from_queryset(individuals_qs):
    # Serialize data from queryset
    individuals, connections = serialize_individuals_and_connections(individuals_qs)

    # Create an empty undirected graph
    G = nx.Graph()

    # Add nodes to the graph with 'immune_system' as a node attribute
    for individual in individuals:
        G.add_node(individual['id'], immune_system=individual['immune_system'])

    # Add edges to the graph with 'weight' as an edge attribute
    for connection in connections:
        G.add_edge(connection['source_id'], connection['target_id'], weight=connection['weight'])

    return G
