from django.test import TestCase
from simulations.models import Pathogen

class PathogenTest(TestCase):
    def setUp(self):
        # Create a pathogen with known point values for testing
        self.pathogen = Pathogen.objects.create(
            infection_rate_points=50,
            death_rate_points=75,
            mutation_rate_points=25,
            incubation_time_points=33,
            infection_time_points=66,
            immunity_duration_points=100,
            days_infected=0,
            attack_vector='101010101'
        )

    def test_infection_rate(self):
        self.assertAlmostEqual(self.pathogen.infection_rate, 0.5, places=3)

    def test_death_rate(self):
        self.assertAlmostEqual(self.pathogen.death_rate, 0.75, places=3)

    def test_mutation_rate(self):
        self.assertAlmostEqual(self.pathogen.mutation_rate, 0.25, places=3)

    def test_incubation_time(self):
        # With the point value of 33, the resulting time should be about 5
        self.assertEqual(self.pathogen.incubation_time, 5)

    def test_infection_time(self):
        # With the point value of 66, the resulting time should be about 10
        self.assertEqual(self.pathogen.infection_time, 10)

    def test_immunity_duration(self):
        # With the point value of 99, the resulting time should be close to the maximum
        self.assertEqual(self.pathogen.immunity_duration, 16)

    def test_immunity_duration_none(self):
        # Test case when immunity_duration_points is None
        self.pathogen.immunity_duration_points = None
        self.assertIsNone(self.pathogen.immunity_duration)