import io
from matplotlib import pyplot as plt
import networkx as nx
from PIL import Image
from tabulate import tabulate


def visualize_population_no_weights(population, save_path=""):
    # Create a NetworkX graph
    G = nx.Graph()

    # Add individuals as nodes
    individuals = population.individuals.all()
    for individual in individuals:
        G.add_node(individual.id)

    # Add connections as edges
    for individual in individuals:
        connections = individual.connections.all()
        for connection in connections:
            G.add_edge(individual.id, connection.id)

    # Draw the graph
    pos = nx.spring_layout(G, seed=42)
    nx.draw(
        G, pos, with_labels=True, node_size=50, node_color="lightblue", font_size=3
    )

    plt.title(f"Population {population.id}: {population.network_topology}")

    # Save the plot as an in-memory binary stream
    buf = io.BytesIO()
    plt.savefig(buf, format="png")
    buf.seek(0)

    # Convert the binary stream to a PIL Image object
    img = Image.open(buf)
    img.save(save_path)
    # Close the buffer
    buf.close()
    # Clear the current figure to free up memory
    plt.clf()

    return img


def create_text_bar_chart(data, bar_width=20, max_height=10, unit_char="█"):
    max_value = max(data.values())

    # Calculate the scaling factor for the chart
    scale = max_value / max_height

    # Find the maximum length of the number values for formatting purposes
    max_number_length = max(len(str(value)) for value in data.values())

    print("Text-based Bar Chart\n")
    for key, value in data.items():
        # Calculate the bar height
        bar_height = int(value / scale)

        # Create the bar using the unit character
        bar = unit_char * bar_height

        # Create a label for the bar, left-justified
        label = f"{key}: ".ljust(bar_width, " ")

        # Format the value with proper alignment
        value_str = str(value).rjust(max_number_length, " ")

        # Print the label followed by the bar and the value
        print(f"{label} {bar} {value_str}")

# Example usage
